@@ -911,7 +911,7 @@ print_version_action (GPParams __unused__ *p)
 		  "\n"
 		  "This version of gphoto2 is using the following software versions and options:\n"),
 		VERSION,
-		2014, /* year of release! */
+		2015, /* year of release! */
 		port_message
 		);
 
@@ -938,8 +938,8 @@ print_version_action (GPParams __unused__ *p)
 	return (GP_OK);
 }
 
-int
-action_camera_capture_preview (GPParams *p)
+static int
+_action_camera_capture_preview (GPParams *p, int viewasciiart)
 {
 	CameraFile *file;
 	int	r, fd;
@@ -961,10 +961,12 @@ action_camera_capture_preview (GPParams *p)
 	}
 
 #ifdef HAVE_AA
-	r = gp_cmd_capture_preview (p->camera, file, p->context);
-#else
-	r = gp_camera_capture_preview (p->camera, file, p->context);
+	if (viewasciiart)
+		r = gp_cmd_capture_preview (p->camera, file, p->context);
+	else
 #endif
+		r = gp_camera_capture_preview (p->camera, file, p->context);
+
 	if (r < 0) {
 		gp_file_unref (file);
 		unlink (tmpname);
@@ -980,6 +982,16 @@ action_camera_capture_preview (GPParams *p)
 	return (GP_OK);
 }
 
+int
+action_camera_capture_preview (GPParams *p) {
+	return _action_camera_capture_preview (p, 0);
+}
+
+int
+action_camera_show_preview (GPParams *p) {
+	return _action_camera_capture_preview (p, 1);
+}
+
 enum moviemode { MOVIE_ENDLESS, MOVIE_FRAMES, MOVIE_SECONDS };
 
 int
@@ -1081,7 +1093,7 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 
 	wp.type = WAIT_EVENTS;
 	wp.u.events = 1000000;
-	if (!arg) {
+	if (!arg || !strlen(arg)) {
 		printf ( _("Waiting for events from camera. Press Ctrl-C to abort.\n"));
 	} else {
 		int x;
@@ -1089,20 +1101,23 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 			wp.type			= WAIT_FRAMES;
 			wp.u.frames		= x;
 			printf ( _("Waiting for %d frames from the camera. Press Ctrl-C to abort.\n"), x);
-		}
+		} else
 		if ((strlen(arg)>2) && (!strcmp(&arg[strlen(arg)-2],"ms")) && sscanf(arg,"%dms",&x)) { /* exact milliseconds */ 
 			wp.type			= WAIT_TIME;
 			wp.u.milliseconds	= x;
 			printf ( _("Waiting for %d milliseconds for events from camera. Press Ctrl-C to abort.\n"), x);
-		}
+		} else
 		if ((wp.type != WAIT_TIME) && (arg[strlen(arg)-1]=='s') && sscanf(arg,"%ds", &x)) { /* exact seconds */ 
 			wp.type			= WAIT_TIME;
 			wp.u.milliseconds	= x*1000;
 			printf ( _("Waiting for %d seconds for events from camera. Press Ctrl-C to abort.\n"), x);
-		}
-		if (wp.type == WAIT_EVENTS) {
+		} else if ((wp.type == WAIT_EVENTS) && sscanf(arg,"%d", &x)) {
 			wp.u.events = atoi(arg);
 			printf ( _("Waiting for %d events from camera. Press Ctrl-C to abort.\n"), wp.u.events);
+		} else {
+			wp.type = WAIT_STRING;
+			wp.u.str = arg;
+			printf ( _("Waiting for %s event from camera. Press Ctrl-C to abort.\n"), wp.u.str);
 		}
 	}
 
@@ -1116,6 +1131,8 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 
 		exitloop = 0;
 		switch (wp.type) {
+		case WAIT_STRING:
+			break;
 		case WAIT_EVENTS:
 			if (events >= wp.u.events) exitloop = 1;
 			break;
@@ -1142,6 +1159,12 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 		case GP_EVENT_UNKNOWN:
 			if (data) {
 				printf("UNKNOWN %s\n", (char*)data);
+				if (wp.type == WAIT_STRING) {
+					if (strstr(data,wp.u.str)) {
+						printf(_("event found, stopping wait!\n"));
+						return GP_OK;
+					}
+				}
 			} else {
 				printf("UNKNOWN\n");
 			}
@@ -1151,14 +1174,26 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 			break;
 		case GP_EVENT_CAPTURE_COMPLETE:
 			printf("CAPTURECOMPLETE\n");
+			if ((wp.type == WAIT_STRING) && strstr("CAPTURECOMPLETE",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
 			break;
 		case GP_EVENT_FILE_ADDED:
 			frames++;
 
 			fn = (CameraFilePath*)data;
 
-			if (downloadtype == DT_NO_DOWNLOAD) {
+			if (	(downloadtype == DT_NO_DOWNLOAD)	||
+				(	(p->flags & FLAGS_KEEP_RAW) &&
+				 	( !strstr(fn->name,".jpg") && !strstr(fn->name,".JPG")) 
+				)
+			) {
 				printf("FILEADDED %s %s\n",fn->name, fn->folder);
+				if ((wp.type == WAIT_STRING) && strstr("FILEADDED",wp.u.str)) {
+					printf(_("event found, stopping wait!\n"));
+					return GP_OK;
+				}
 				continue;
 			}
 			/* Otherwise download the image and continue... */
@@ -1193,10 +1228,18 @@ action_camera_wait_event (GPParams *p, enum download_type downloadtype, const ch
 					/* dont continue in event loop */
 				}
 			}
+			if ((wp.type == WAIT_STRING) && strstr("FILEADDED",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
 			break;
 		case GP_EVENT_FOLDER_ADDED:
 			fn = (CameraFilePath*)data;
 			printf("FOLDERADDED %s %s\n",fn->name, fn->folder);
+			if ((wp.type == WAIT_STRING) && strstr("FOLDERADDED",wp.u.str)) {
+				printf(_("event found, stopping wait!\n"));
+				return GP_OK;
+			}
 			break;
 		}
 		free (data);
@@ -1346,14 +1389,15 @@ debug_func (GPLogLevel level, const char *domain, const char *str, void *data)
 	sec = tv.tv_sec  - glob_tv_zero.tv_sec;
 	usec = tv.tv_usec - glob_tv_zero.tv_usec;
 	if (usec < 0) {sec--; usec += 1000000L;}
-	fprintf (logfile, "%li.%06li %s(%i): %s\n", sec, usec, domain, level, str);
+	fprintf (logfile, "%li.%06li %-28s(%i): %s\n", sec, usec, domain, level, str);
 }
 
 int
-debug_action (GPParams *p, const char *debug_logfile_name)
+debug_action (GPParams *p, const char *debug_loglevel, const char *debug_logfile_name)
 {
 	int n;
 	FILE *logfile = NULL;
+	GPLogLevel loglevel = GP_LOG_ALL;
 
 	/* make sure we're only executed once */
 	static int debug_flag = 0;
@@ -1361,6 +1405,15 @@ debug_action (GPParams *p, const char *debug_logfile_name)
 		return(GP_OK);
 	debug_flag = 1;
 
+	if (debug_loglevel && !strcmp(debug_loglevel, "error"))
+		loglevel = GP_LOG_ERROR;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "debug"))
+		loglevel = GP_LOG_DEBUG;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "data"))
+		loglevel = GP_LOG_DATA;
+	else if (debug_loglevel && !strcmp(debug_loglevel, "all"))
+		loglevel = GP_LOG_ALL;
+
 	if (debug_logfile_name != NULL) {
 	  /* FIXME: Handle fopen() error besides using stderr? */
 	  logfile = fopen(debug_logfile_name, "a");
@@ -1373,7 +1426,7 @@ debug_action (GPParams *p, const char *debug_logfile_name)
 
 	gettimeofday (&glob_tv_zero, NULL);
 
-	CR (p->debug_func_id = gp_log_add_func (GP_LOG_ALL, debug_func, (void *) logfile));
+	CR (p->debug_func_id = gp_log_add_func (loglevel, debug_func, (void *) logfile));
 	gp_log (GP_LOG_DEBUG, "main", _("ALWAYS INCLUDE THE FOLLOWING LINES "
 					"WHEN SENDING DEBUG MESSAGES TO THE "
 					"MAILING LIST:"));
@@ -1619,6 +1672,7 @@ print_widget (GPParams *p, const char *name, CameraWidget *widget) {
 		printf ("Type: DATE\n");
 		printf ("Current: %d\n", t);
 		printf ("Printable: %s\n", timebuf);
+		printf ("Help: %s\n", _("Use 'now' as the current time when setting.\n"));
 		break;
 	}
 	case GP_WIDGET_MENU:
@@ -1750,21 +1804,31 @@ set_config_action (GPParams *p, const char *name, const char *value) {
 		break;
 	}
 	case GP_WIDGET_DATE:  {		/* int			*/
-		int	t = -1;
+		time_t	t = -1;
 		struct tm xtm;
 
+		memset(&xtm,0,sizeof(xtm));
+
+		/* We need to set UNIX time in seconds since Epoch */
+		/* We get ... local time */
+
 		if (!strcasecmp (value, "now")	|| !strcasecmp (value, _("now")))
 			t = time(NULL);
 #ifdef HAVE_STRPTIME
-		else if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm))
+		else if (strptime (value, "%c", &xtm) || strptime (value, "%Ec", &xtm)) {
+			xtm.tm_isdst = -1;
 			t = mktime (&xtm);
+		}
 #endif
 		if (t == -1) {
-			if (!sscanf (value, "%d", &t)) {
+			unsigned long lt;
+
+			if (!sscanf (value, "%ld", &lt)) {
 				gp_context_error (p->context, _("The passed value %s is neither a valid time nor an integer."), value);
 				ret = GP_ERROR_BAD_PARAMETERS;
 				break;
 			}
+			t = lt;
 		}
 		ret = gp_widget_set_value (child, &t);
 		if (ret != GP_OK)
