@@ -1,6 +1,6 @@
 /*
- * Copyright 2002 Lutz Mller <lutz@users.sourceforge.net>
- * Copyright 2004-2010 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright 2002 Lutz Mueller <lutz@users.sourceforge.net>
+ * Copyright 2004-2013 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -149,19 +149,21 @@ get_path_for_file (const char *folder, const char *name, CameraFileType type, Ca
 {
 	unsigned int i, l;
 	char *s, b[1024];
-	time_t t;
+	time_t t = 0;
 	struct tm *tm;
 	int hour12;
 	static int filenr = 1;
 
-	if (!file || !path)
+	if (!path)
 		return (GP_ERROR_BAD_PARAMETERS);
 
 	*path = NULL;
-	CR (gp_file_get_mtime (file, &t));
+	if (file) {
+		CR (gp_file_get_mtime (file, &t));
 	
-	if (!t)	/* use the current time as fallback if the camera did not return it. */
-		t = time(NULL);
+		if (!t)	/* use the current time as fallback if the camera did not return it. */
+			t = time(NULL);
+	}
 
 	tm = localtime (&t);
 	hour12 = tm->tm_hour % 12;
@@ -173,8 +175,17 @@ get_path_for_file (const char *folder, const char *name, CameraFileType type, Ca
 	 * If the user didn't specify a filename, use the original name 
 	 * (and prefix).
 	 */
-	if (!gp_params.filename || !strcmp (gp_params.filename, ""))
-		return gp_file_get_name_by_type (file, name, type, path);
+	if (!gp_params.filename || !strcmp (gp_params.filename, "")) {
+		if (file) {
+			return gp_file_get_name_by_type (file, name, type, path);
+		} else if ((type == GP_FILE_TYPE_NORMAL) && strchr(name,'.')) {
+			/* return the original name */
+			*path = strdup(name);
+			return (GP_OK);
+		} else
+			/* Download required to get the path from CameraFile *file */
+			return (GP_ERROR_BAD_PARAMETERS);
+	}
 
 	/* The user did specify a filename. Use it. */
 	b[sizeof (b) - 1] = '\0';
@@ -275,6 +286,18 @@ get_path_for_file (const char *folder, const char *name, CameraFileType type, Ca
 				}
 				break;
 
+			case 'F':
+				/* Get the folder name */
+				{
+					const char *f = folder ? folder : "";
+					if (f[0] == '/') f++; /* Skip first '/' */
+					if (!f[0]) f = "."; /* replace empty folder name by '.' */
+
+					strncpy (b, f, sizeof (b) - 1 - strlen(b));
+					b[sizeof(b) - 1] = '\0';
+				}
+				break;
+
 			case 'a':
 			case 'A':
 			case 'b':
@@ -292,6 +315,8 @@ get_path_for_file (const char *folder, const char *name, CameraFileType type, Ca
 			case 'Y':
 				{
 					char fmt[3] = { '%', '\0', '\0' };
+					if (!file) return (GP_ERROR_BAD_PARAMETERS); /* mtime unknown */
+
 					fmt[1] = gp_params.filename[i]; /* the letter of this 'case' */
 					strftime(b, sizeof (b), fmt, tm);
 					break;
@@ -352,6 +377,15 @@ save_camera_file_to_file (
 	free (path);
 	path = NULL;
 
+        if ((gp_params.flags & FLAGS_SKIP_EXISTING) && gp_system_is_file (s)) {
+		if ((gp_params.flags & FLAGS_QUIET) == 0) {
+			printf (_("Skip existing file %s\n"), s);
+			fflush (stdout);
+		}
+		if (curname)
+			unlink (curname);
+		return (GP_OK);
+	}
         if ((gp_params.flags & FLAGS_QUIET) == 0) {
                 while ((gp_params.flags & FLAGS_FORCE_OVERWRITE) == 0 &&
 		       gp_system_is_file (s)) {
@@ -401,8 +435,32 @@ save_camera_file_to_file (
 		int x;
 
 		unlink(s);
-		if (-1 == rename (curname, s))
-			perror("rename");
+		if (-1 == rename (curname, s)) {
+			/* happens if the user specified a absolute path with --filename */
+			if (errno == EXDEV) {
+				char buf[8192];
+				int in_fd, out_fd;
+
+				in_fd = open(curname, O_RDONLY);
+				if (in_fd < 0)
+					perror("Can't open file for reading");
+
+				out_fd = open(s, O_CREAT | O_WRONLY, 0644);
+				if (out_fd < 0)
+					perror("Can't open file for writing");
+
+				while (1) {
+					ssize_t result = read(in_fd, buf, sizeof(buf));
+					if (!result) break;
+					if (-1 == write(out_fd, buf, result)) {
+						perror("write");
+						break;
+					}
+				}
+				unlink(curname);
+			} else
+				perror("rename");
+		}
 		x = umask(0022); /* get umask */
 		umask(x);/* set it back to the old value */
 		chmod(s,0666 & ~x);
@@ -515,6 +573,22 @@ save_file_to_file (Camera *camera, GPContext *context, Flags flags,
 	char	tmpname[20], *tmpfilename;
 	struct privstr *ps = NULL;
 
+	if (flags & FLAGS_SKIP_EXISTING && !(flags & FLAGS_STDOUT)) {
+		char *path = NULL;
+		/* Check if the file is present before downloading it. */
+		res = get_path_for_file (folder, filename, type, NULL, &path);
+
+		if (res == (GP_OK) && gp_system_is_file (path)) {
+			/* File name pattern do not require CameraFile and target file
+			   exists: Skip this file. */
+			if ((gp_params.flags & FLAGS_QUIET) == 0) {
+				printf (_("Skip existing file %s\n"), path);
+				fflush (stdout);
+			}
+			return (GP_OK);
+		}
+	}
+
 	if (flags & FLAGS_NEW) {
 		CameraFileInfo info;
 		
@@ -751,6 +825,14 @@ save_captured_file (CameraFilePath *path, int download) {
 				return (result);
 			}
 		}
+		if (	(gp_params.flags & FLAGS_KEEP_RAW) &&
+			(!strstr(path->name,".jpg") && !strstr(path->name,".JPG"))
+		) {
+			if (!(gp_params.flags & FLAGS_QUIET))
+				printf (_("Keeping file %s%s%s on the camera\n"),
+					path->folder, pathsep, path->name);
+			return GP_OK;
+		}
 
 		result = get_file_common (path->name, GP_FILE_TYPE_NORMAL);
 		if (result != GP_OK) {
@@ -909,12 +991,15 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 			 */
 		} else {
 			result = GP_ERROR_NOT_SUPPORTED;
+#if 0
+			/* Not a good idea, as we do not know how long to wait after capture ... */
 			if (a.operations & GP_OPERATION_TRIGGER_CAPTURE) {
 				result = gp_camera_trigger_capture (gp_params.camera, gp_params.context);
 				if ((result != GP_OK) && (result != GP_ERROR_NOT_SUPPORTED))
 					cli_error_print(_("Could not trigger image capture."));
 				/* The downloads will be handled by wait_event */
 			}
+#endif
 			if (result == GP_ERROR_NOT_SUPPORTED) {
 				result = gp_camera_capture (gp_params.camera, type, &path, gp_params.context);
 				if (result != GP_OK) {
@@ -941,6 +1026,7 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 					(result == GP_ERROR_NO_MEMORY)		||
 					(result == GP_ERROR_CANCEL)		||
 					(result == GP_ERROR_NO_SPACE)		||
+					(result == GP_ERROR_IO_USB_CLAIM)	||
 					(result == GP_ERROR_OS_FAILURE)
 				)
 					return (result);
@@ -1024,15 +1110,26 @@ capture_generic (CameraCaptureType type, const char __unused__ *name, int downlo
 	 * so make sure we wait a bit for the the camera to finish stuff.
 	 */
 	gettimeofday (&expose_end_time, NULL);
-	waittime = 100;
-	if (glob_frames || end_next || !glob_interval || glob_bulblength) waittime = 2000;
+	waittime = 3000;
+	/* tricky, this loop might need to download both JPG and RAW. so wait longer. */
+	/*if (glob_frames || end_next || !glob_interval || glob_bulblength) waittime = 2000;*/
 	/* Drain the event queue at the end and download left over added images */
-	while ((-timediff_now(&expose_end_time)) < waittime) {
-		result = wait_and_handle_event(waittime - (-timediff_now(&expose_end_time)), &evtype, download);
-		if ((result != GP_OK) || (evtype == GP_EVENT_TIMEOUT))
+	while (1) {
+		int realwait = waittime - (-timediff_now(&expose_end_time));
+		if (realwait < 0) realwait = 0; /* just drain the queue now */
+		result = wait_and_handle_event(realwait, &evtype, download);
+		if ((result != GP_OK) || (evtype == GP_EVENT_TIMEOUT)) {
+			/*printf("Timeout or error, leaving loop.\n");*/
 			break;
-		if (evtype == GP_EVENT_CAPTURE_COMPLETE)
+		}
+		if (evtype == GP_EVENT_CAPTURE_COMPLETE) {
+			/*printf("Capture complete, waiting final 0.1s.\n");*/
 			waittime = 100;
+		}
+		if (evtype == GP_EVENT_FILE_ADDED) { /* Restart timer, more image data might come. */
+			/*printf("File added, waiting more %gs.\n", waittime/1000.0);*/
+			gettimeofday (&expose_end_time, NULL);
+		}
 	}
 
 	signal(SIGUSR1, SIG_DFL);
@@ -1176,11 +1273,12 @@ typedef enum {
 	ARG_CAPTURE_IMAGE_AND_DOWNLOAD,
 	ARG_CAPTURE_MOVIE,
 	ARG_CAPTURE_PREVIEW,
+	ARG_SHOW_PREVIEW,
 	ARG_CAPTURE_SOUND,
 	ARG_CAPTURE_TETHERED,
-	ARG_CAPTURE_TETHERED_KEEP,
 	ARG_CONFIG,
 	ARG_DEBUG,
+	ARG_DEBUG_LOGLEVEL,
 	ARG_DEBUG_LOGFILE,
 	ARG_DELETE_ALL_FILES,
 	ARG_DELETE_FILE,
@@ -1204,6 +1302,7 @@ typedef enum {
 	ARG_HELP,
 	ARG_HOOK_SCRIPT,
 	ARG_KEEP,
+	ARG_KEEP_RAW,
 	ARG_LIST_CAMERAS,
 	ARG_LIST_ALL_CONFIG,
 	ARG_LIST_CONFIG,
@@ -1226,6 +1325,7 @@ typedef enum {
 	ARG_SHELL,
 	ARG_SHOW_EXIF,
 	ARG_SHOW_INFO,
+	ARG_SKIP_EXISTING,
 	ARG_SPEED,
 	ARG_STDOUT,
 	ARG_STDOUT_SIZE,
@@ -1345,6 +1445,10 @@ cb_arg_init (poptContext __unused__ ctx,
 	case ARG_NEW:
 		gp_params.flags |= FLAGS_NEW;
 		break;
+	case ARG_KEEP_RAW:
+		gp_params.flags |= FLAGS_KEEP_RAW;
+		break;
+
 	case ARG_KEEP:
 		gp_params.flags |= FLAGS_KEEP;
 		break;
@@ -1369,6 +1473,11 @@ cb_arg_init (poptContext __unused__ ctx,
 			"option ('%s')...", arg);
 		params->p.r = action_camera_set_port (&gp_params, arg);
 		break;
+
+	case ARG_SKIP_EXISTING:
+		gp_params.flags |= FLAGS_SKIP_EXISTING;
+		break;
+
 	case ARG_SPEED:
 		params->p.r = action_camera_set_speed (&gp_params, atoi (arg));
 		break;
@@ -1463,6 +1572,9 @@ cb_arg_run (poptContext __unused__ ctx,
 	case ARG_CAPTURE_PREVIEW:
 		params->p.r = action_camera_capture_preview (&gp_params);
 		break;
+	case ARG_SHOW_PREVIEW:
+		params->p.r = action_camera_show_preview (&gp_params);
+		break;
 	case ARG_CAPTURE_SOUND:
 		params->p.r = capture_generic (GP_CAPTURE_SOUND, arg, 0);
 		break;
@@ -1828,9 +1940,9 @@ main (int argc, char **argv, char **envp)
 {
 	CallbackParams cb_params;
 	poptContext ctx;
-	int help_option_given = 0;
+	int i, help_option_given = 0;
 	int usage_option_given = 0;
-	char *debug_logfile_name = NULL;
+	char *debug_logfile_name = NULL, *debug_loglevel = NULL;
 	const struct poptOption generalOptions[] = {
 		GPHOTO2_POPT_CALLBACK
 		{"help", '?', POPT_ARG_NONE, (void *) &help_option_given, ARG_HELP,
@@ -1839,6 +1951,8 @@ main (int argc, char **argv, char **envp)
 		 N_("Print short message on program usage"), NULL},
 		{"debug", '\0', POPT_ARG_NONE, (void *) &debug_option_given, ARG_DEBUG,
 		 N_("Turn on debugging"), NULL},
+		{"debug-loglevel", '\0', POPT_ARG_STRING, (void *) &debug_loglevel, ARG_DEBUG_LOGLEVEL,
+		 N_("Set debug level [error|debug|data|all]"), NULL},
 		{"debug-logfile", '\0', POPT_ARG_STRING, (void *) &debug_logfile_name, ARG_DEBUG_LOGFILE,
 		 N_("Name of file to write debug info to"), N_("FILENAME")},
 		{"quiet", '\0', POPT_ARG_NONE, NULL, ARG_QUIET,
@@ -1869,7 +1983,7 @@ main (int argc, char **argv, char **envp)
 		{"list-ports", '\0', POPT_ARG_NONE, NULL, ARG_LIST_PORTS,
 		 N_("List supported port devices"), NULL},
 		{"abilities", 'a', POPT_ARG_NONE, NULL, ARG_ABILITIES,
-		 N_("Display camera/driver abilities"), NULL},
+		 N_("Display the camera/driver abilities in the libgphoto2 database"), NULL},
 		POPT_TABLEEND
 	};
 	const struct poptOption configOptions[] = {
@@ -1898,15 +2012,20 @@ main (int argc, char **argv, char **envp)
 		GPHOTO2_POPT_CALLBACK
 		{"keep", '\0', POPT_ARG_NONE, NULL, ARG_KEEP,
 		 N_("Keep images on camera after capturing"), NULL},
+		{"keep-raw", '\0', POPT_ARG_NONE, NULL, ARG_KEEP_RAW,
+		 N_("Keep RAW images on camera after capturing"), NULL},
 		{"no-keep", '\0', POPT_ARG_NONE, NULL, ARG_NO_KEEP,
 		 N_("Remove images from camera after capturing"), NULL},
 		{"wait-event", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL, ARG_WAIT_EVENT,
-		 N_("Wait for event(s) from camera"), N_("COUNT")},
+		 N_("Wait for event(s) from camera"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		{"wait-event-and-download", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
-		 ARG_CAPTURE_TETHERED, N_("Wait for event(s) from the camera and download new images"), N_("COUNT")},
+		 ARG_CAPTURE_TETHERED, N_("Wait for event(s) from the camera and download new images"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		{"capture-preview", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_PREVIEW,
 		 N_("Capture a quick preview"), NULL},
+		{"show-preview", '\0', POPT_ARG_NONE, NULL,
+		 ARG_SHOW_PREVIEW,
+		 N_("Show a quick preview as Ascii Art"), NULL},
 		{"bulb", 'B', POPT_ARG_INT, NULL, ARG_CAPTURE_BULB,
 		 N_("Set bulb exposure time in seconds"), N_("SECONDS")},
 		{"frames", 'F', POPT_ARG_INT, NULL, ARG_CAPTURE_FRAMES,
@@ -1926,7 +2045,7 @@ main (int argc, char **argv, char **envp)
 		{"capture-sound", '\0', POPT_ARG_NONE, NULL,
 		 ARG_CAPTURE_SOUND, N_("Capture an audio clip"), NULL},
 		{"capture-tethered", '\0', POPT_ARG_STRING|POPT_ARGFLAG_OPTIONAL, NULL,
-		 ARG_CAPTURE_TETHERED, N_("Wait for shutter release on the camera and download"), N_("COUNT")},
+		 ARG_CAPTURE_TETHERED, N_("Wait for shutter release on the camera and download"), N_("COUNT, SECONDS, MILLISECONDS or MATCHSTRING")},
 		{"trigger-capture", '\0', POPT_ARG_NONE, NULL,
 		 ARG_TRIGGER_CAPTURE, N_("Trigger image capture"), NULL},
 		POPT_TABLEEND
@@ -1988,6 +2107,8 @@ main (int argc, char **argv, char **envp)
 		 N_("Process new files only"), NULL},
 		{"force-overwrite", '\0', POPT_ARG_NONE, NULL,
 		 ARG_FORCE_OVERWRITE, N_("Overwrite files without asking"), NULL},
+		{"skip-existing", '\0', POPT_ARG_NONE, NULL,
+		 ARG_SKIP_EXISTING, N_("Skip existing files"), NULL},
 		POPT_TABLEEND
 	};
 	const struct poptOption miscOptions[] = {
@@ -2001,12 +2122,12 @@ main (int argc, char **argv, char **envp)
 
 #ifdef HAVE_LIBEXIF
 		{"show-exif", '\0', POPT_ARG_STRING, NULL, ARG_SHOW_EXIF,
-		 N_("Show EXIF information"), NULL},
+		 N_("Show EXIF information of JPEG images"), NULL},
 #endif
 		{"show-info", '\0', POPT_ARG_STRING, NULL, ARG_SHOW_INFO,
-		 N_("Show info"), NULL},
+		 N_("Show image information, like width, height, and capture time"), NULL},
 		{"summary", '\0', POPT_ARG_NONE, NULL, ARG_SUMMARY,
-		 N_("Show summary"), NULL},
+		 N_("Show camera summary"), NULL},
 		{"manual", '\0', POPT_ARG_NONE, NULL, ARG_MANUAL,
 		 N_("Show camera driver manual"), NULL},
 		{"about", '\0', POPT_ARG_NONE, NULL, ARG_ABOUT,
@@ -2091,9 +2212,14 @@ main (int argc, char **argv, char **envp)
 		return 0;
 	}
 	if (debug_option_given) {
-		CR_MAIN (debug_action (&gp_params, debug_logfile_name));
+		CR_MAIN (debug_action (&gp_params, debug_loglevel, debug_logfile_name));
 	}
 
+	gp_log (GP_LOG_DEBUG, "main", "invoked with following arguments:");
+	for (i=1;i<argc;i++)
+		gp_log (GP_LOG_DEBUG, "main", "  %s", argv[i]);
+
+
 	/* Initialize. */
 #ifdef HAVE_PTHREAD
 	gp_camera_set_timeout_funcs (gp_params.camera, start_timeout_func,
@@ -2149,9 +2275,9 @@ main (int argc, char **argv, char **envp)
 	CHECK_OPT (ARG_CAPTURE_IMAGE_AND_DOWNLOAD);
 	CHECK_OPT (ARG_CAPTURE_MOVIE);
 	CHECK_OPT (ARG_CAPTURE_PREVIEW);
+	CHECK_OPT (ARG_SHOW_PREVIEW);
 	CHECK_OPT (ARG_CAPTURE_SOUND);
 	CHECK_OPT (ARG_CAPTURE_TETHERED);
-	CHECK_OPT (ARG_CAPTURE_TETHERED_KEEP);
 	CHECK_OPT (ARG_CONFIG);
 	CHECK_OPT (ARG_DELETE_ALL_FILES);
 	CHECK_OPT (ARG_DELETE_FILE);
@@ -2339,6 +2465,7 @@ main (int argc, char **argv, char **envp)
 	}
 
         signal (SIGINT, signal_exit);
+        signal (SIGTERM, signal_exit);
 
 	/* If we are told to be quiet, be so. *
 	cb_params.type = CALLBACK_PARAMS_TYPE_QUERY;
